from enum import Enum

class PyLevel(Enum):
    Trace: PyLevel
    Debug: PyLevel
    Info: PyLevel
    Warn: PyLevel
    Error: PyLevel

class PyLogger:
    def __init__(
        self, name: str, path: str | None = None, level: PyLevel = PyLevel.Info
    ) -> None: ...
    def shutdown(self) -> None: ...
    def trace(self, message: str) -> None: ...
    def debug(self, message: str) -> None: ...
    def info(self, message: str) -> None: ...
    def warn(self, message: str) -> None: ...
    def error(self, message: str) -> None: ...

def basic_config(path: str | None = None) -> None: ...
def get_logger(name: str, level: PyLevel = PyLevel.Info) -> PyLogger: ...
